"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class InternalGraph {
    constructor() {
        this._matrix = new Map();
        this._parents = new Map();
    }
    link(from, to) {
        this._populateAdjacencyMapping(this._matrix, from, to);
        this._populateAdjacencyMapping(this._parents, to, from);
    }
    _populateAdjacencyMapping(adjacencyMap, from, to) {
        if (!adjacencyMap.has(from)) {
            adjacencyMap.set(from, new Set());
        }
        const linkedVertexes = adjacencyMap.get(from);
        linkedVertexes.add(to);
    }
    add(vertex) {
        if (this._matrix.has(vertex)) {
            return;
        }
        this._matrix.set(vertex, new Set());
    }
    ancestorsOf(vertex) {
        const searchState = this._setupSearchState();
        searchState.unmarked.delete(vertex);
        this._visit(vertex, this._parents, searchState);
        return searchState.l.reverse();
    }
    topologicalSort() {
        const searchState = this._setupSearchState();
        while (searchState.unmarked.size > 0) {
            const unmarkedNode = searchState.unmarked.values().next().value;
            searchState.unmarked.delete(unmarkedNode);
            this._visit(unmarkedNode, this._matrix, searchState);
        }
        return searchState.l;
    }
    _visit(vertex, adjacencyMap, searchState) {
        if (searchState.marked.has(vertex)) {
            return;
        }
        if (searchState.tempMarked.has(vertex)) {
            throw new Error('Not a dag');
        }
        searchState.tempMarked.add(vertex);
        for (const child of adjacencyMap.get(vertex) || new Set()) {
            this._visit(child, adjacencyMap, searchState);
        }
        searchState.tempMarked.delete(vertex);
        searchState.unmarked.delete(vertex);
        searchState.marked.add(vertex);
        searchState.l.push(vertex);
    }
    _setupSearchState() {
        const allVertexes = new Set(Array.from(this._matrix.keys()).concat(Array.from(this._matrix.values()).reduce((acc, s) => acc.concat(Array.from(s)), [])));
        const searchState = {
            marked: new Set(),
            tempMarked: new Set(),
            unmarked: allVertexes,
            l: []
        };
        return searchState;
    }
}
exports.default = InternalGraph;
//# sourceMappingURL=internalGraph.js.map