import { IMerklingInternalRecord, IMerklingProxyState, MerklingProxyRef } from './merklingProxyHandler';
import IpfsWrapper from './ipfsWrapper';
import Serialiser from './serialiser';
import InternalGraph from './internalGraph';
import { IIpfsNode, ICid } from './domain';
interface IInternalisedStateResult {
    state: any;
    childInternalIds: number[];
}
export default class MerklingSession {
    /** @private the IPFS node */
    _ipfs: IpfsWrapper;
    /** @private the internal counter of IPLD blocks loaded into the session */
    _ipldIdCounter: number;
    /** @private the identity map and internal record keeping of the loaded IPLD blocks */
    _ipldNodeEntries: Map<number, IMerklingInternalRecord>;
    /** @private a mapping proxies that exist in the session */
    _proxies: Map<string, {}>;
    /** @private the serialiser and deserialization layer on going to and from IPFS */
    _serialiser: Serialiser;
    /** @private the internal graph of IPLD block to IPLD block relationships */
    _internalGraph: InternalGraph;
    constructor({ ipfs }: {
        ipfs: IIpfsNode;
    });
    /**
     * Create a new dirty Merkling proxy that can be manipulated
     * before later being persisted.
     *
     * @param objState a js object of the state for the IPLD block
     * @returns a proxy representing the given state as an IPLD block
     */
    create<T>(objState: T): T;
    /**
     * Get an IPLD block from IPFS and return
     * it as a tracked Merkling proxy.
     * @param hashOrCid a CID or base encoded version
     * @returns a clean proxy with the state loaded and accessible
     */
    get(hashOrCid: string | ICid): Promise<any>;
    load(hash: string | ICid): IMerklingProxyState;
    /**
     * Save all proxies that are currently marked as dirty,
     * and any proxies that depend on them.
     */
    save(): Promise<void>;
    /**
     * @private Given a merkling ref, retreive its state
     * from IPFS and update the internal record.
     *
     * @param ref a proxy ref to resolve
     * @returns a promise complete with the retrieval
     */
    _resolveRef(ref: MerklingProxyRef): Promise<void>;
    /**
     * @private Given an internal record mark it as dirty and
     * all loaded ancestors in the graph.
     *
     * @param internalId
     */
    _markRecordAndAncestorsAsDirty(internalId: number): void;
    _internaliseState(obj: any): IInternalisedStateResult;
    private _loadCidIntoSession;
    private _addIpldNodeEntry;
    private _recursiveSaveIpldNode;
    private _saveRecordToIpfs;
    private _readStateFromIpfs;
    private _toCid;
}
export {};
