"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable @typescript-eslint/no-explicit-any */
const cids_1 = __importDefault(require("cids"));
const symbols_1 = require("./symbols");
class Serialiser {
    constructor(internalIdToCidEncoder) {
        this._internalIdToCidEncoder = internalIdToCidEncoder;
    }
    serialise(obj) {
        return this._cloneAndSubstitute(obj, this._merkleRefToIpldRef.bind(this));
    }
    deserialise(obj, internalIpldNodeMinter) {
        return this._cloneAndSubstitute(obj, this._merkleRefToIpldRefBuilder(internalIpldNodeMinter));
    }
    internalise(obj, internalLinker) {
        return this._cloneAndSubstitute(obj, this._merkleProxyToMerkleRefBuilder(internalLinker));
    }
    _merkleRefToIpldRefBuilder(internalIpldNodeMinter) {
        return (obj, key, v) => {
            if (cids_1.default.isCID(v)) {
                obj[key] = internalIpldNodeMinter(v);
                return true;
            }
            else {
                return false;
            }
        };
    }
    _merkleRefToIpldRef(obj, key, v) {
        if ('isRef' in v && v.isRef) {
            const ref = v;
            obj[key] = this._internalIdToCidEncoder(ref.internalId);
            return true;
        }
        else {
            return false;
        }
    }
    _merkleProxyToMerkleRefBuilder(internalLinker) {
        return (obj, key, v) => {
            if (v[symbols_1.isProxySymbol]) {
                const ref = v[symbols_1.getRefSymbol];
                obj[key] = ref;
                internalLinker(ref.internalId);
                return true;
            }
            else {
                return false;
            }
        };
    }
    _cloneAndSubstitute(obj, substitution) {
        var clone = Array.isArray(obj) ? [] : {};
        for (var i in obj) {
            if (obj[i] != null && typeof obj[i] === 'object') {
                if (!substitution(clone, i, obj[i])) {
                    clone[i] = this._cloneAndSubstitute(obj[i], substitution);
                }
            }
            else {
                clone[i] = obj[i];
            }
        }
        return clone;
    }
}
exports.default = Serialiser;
//# sourceMappingURL=serialiser.js.map